## Memory Access Analysis Design draft
# Analysis goals
* Track down memory writes transparently along the call graph
* Provide handy client query interface
* Use fine grain memory map based on the GEPs


# Analysis interface
#### local accesses  
`SetVector<uint16_t> & getLocalAccessList(const Value * ptr)`    

##### input arguments
> **_ptr_** is treated as a base for the local writes query.  

##### return values  
* **_Vector of the offsets_** related to *_ptr_* which is written inside the current Function.
  + **NOTE:** current Function is defined based on the input argument `ptr->getParent()->getParent()`
* **`nullptr`** if the input parameter is not a valid Argument  
                (_i.e. formal argument of some Function in current Module_) or GEP based on the valid Argument
* **_Vector of 1 element with special value_** - __MA_UNKNOWN__ designates that inside the Function
   there is a writes over the GEP with at least one non constant index  
   in this case we consider that given base pointer might be written at any offset.  

##### example  

>```void foo(__global int * p) 
{    
          p[2] = 123; 
          p[5] = 0;
}```


>`__kernel void test(__global int * in)  
{  
  foo(in + 4);  
}`

> `getLocalAccessList(in)` returns empty vector
> `getLocalAccessList(p)` returns: 2, 5  
>  We'd assume that  
>  `getLocalAccessList(p+4)` returns: 6, 9  
>  but it returns **nullptr** instead  
>  because **__p+4__** is not a valid GEP in the foo's scope  
>  we'd use another query instead


`SetVector<uint16_t> & getLocalAccessList(const Value * actual, const Value * formal)`    

##### input arguments
> **_actual_** is treated as a base for the writes query which is local to the Function  
> to which corresponding **_formal_** belongs.

##### return values  
* **_Vector of the offsets_** related to *_actual_* which is written inside the Function  
> corresponding to **_formal_**.  
* **`nullptr`** if **_actual_** does not map to a valid **_formal_**
* **_Vector of 1 element with special value_** - __MA_UNKNOWN__ designates that inside the Function
   there is a writes over the GEP with at least one non constant index  
   in this case we consider that given base pointer might be written at any offset.  
   
##### example  

>```void foo(__global int * p) 
{    
          p[2] = 123; 
          p[5] = 0;
}```


>`__kernel void test(__global int * in)  
{  
  foo(in + 4);  
}`

>  `getLocalAccessList(in+4, p)` returns: 6, 9

#### global accesses  
`SetVector<uint16_t> & getGlobalAccessList(const Value * ptr)`
##### input arguments
> **_ptr_** is treated as a base for the global writes query.  
##### return values  
* **_Vector of the offsets_** related to *_ptr_* which is written inside the current Module.  
> Writes via non constant GEP indexes are indicated with __MA_UNKNOWN__ elements  
* **`nullptr`** if the input parameter is not a valid Argument  
                (_i.e. formal argument of some Function in current Module_) or GEP based on the valid Argument in any Function in the current Module  


##### example  

>```void foo(__global int * p) 
{    
          p[2] = 123; 
          p[5] = 0;
}```


>`__kernel void test(__global int * in, int n)  
{  
  in[3] = 123;
  in[n+5] = 0;
  foo(in + 4);  
}`

> `getGlobalAccessList(p)` 
> assuming that `p = in + 4`  writes to `in[3]` is out of queried range so the only write from the outer scope is `in[n+5]` that may really write to any location for `n > -1`
> returns: 2, 5, MA_UNKNOWN  

# Structures and algorithms.

#### Basic data structures

* The core structure is a mapping of each pointer formal or actual argument to it's access lists.  
  * `DenseMap<const Value *, struct MA_ENTRY *>`  
* MA_WRITE describes individual write to the memory location as offset from the parent pointer and width  

    struct MA\_WRITE {  
    > struct MA\_WRITE * next;  
    > struct MA\_WRITE * prev;  
    > uint16_t width;  
    > uint16_t offset;  
    
    }  
    

* MA_ENTRY is the value member of the  
  `DenseMap<const Value *, struct MA_ENTRY *>`

    struct MA\_ENTRY {  
    > struct MA\_WRITE * local;    // double linked list of MA\_WRITE for the writes local to the Function corresponding to the entry pointer value  
    > struct MA\_WRITE * global;   // same for global accesses  
    > struct MA\_ENTRY * parent;   // parent for the pointer value in the call graph  
    > SetVector \<struct MA\_ENTRY *\> children; // children of the current pointer in the call graph  
    
    }  
    
* All the writes inside the callee body are considered **_local_** for any caller pointer passed as actual argument.  
* Caller's actual local access list is implicitly merged with all the callee local access lists by means of the walk through the children list.  
* Any additional offsets that originated from the GEP indices of the actual argument are applied transparently.  
    
##### example

Let's consider the following code:

    void foo(__global int *p) {  
      p[2] = 1;
      p[4] = 2;
    }
    
    void bar(__global int * q) {  
      q[3] = 10;
    }
    
    __kernel void test(__global int * in) {  
      in[0] = 0;  
      foo(in);  
      bar(in+2);  
    }


